==> audiochain-1.1.5-src.tgz: org/audiochain/model/BinaryAudioProject.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import static org.audiochain.model.ObjectStore.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.TreeSet;


public final class BinaryAudioProject implements BinaryDataset, RemoteBinaryDatasetProvider, Serializable, Comparator<BinaryDataset> {
	private static final long serialVersionUID = 1L;
	private final AudioProject audioProject;
	private transient long length;
	private transient boolean lengthComputed;
	private transient Collection<BinaryDataset> additionalBinaryDatasets;
	private Collection<String> remoteAvailableBinaryDatasetIdentifiers;

	public BinaryAudioProject() {
		// for deserialization only
		audioProject = null;
	}

	public BinaryAudioProject(AudioProject audioProject) {
		this.audioProject = audioProject;
	}

	@Override
	public AudioProject getBinaryDatasetAssociatedAudioProject() {
		return audioProject;
	}

	@Override
	public String getBinaryDatasetIdentifier() {
		return audioProject.getIdentifier();
	}

	@Override
	public InputStream getBinaryDatasetInputStream() throws IOException {
		return createInputStream();
	}

	private InputStream createInputStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		remoteAvailableBinaryDatasetIdentifiers = new HashSet<String>();
		Collection<BinaryDataset> allBinaryDatasets = audioProject.getAllBinaryDatasets();
		for (BinaryDataset binaryDataset : allBinaryDatasets) {
			remoteAvailableBinaryDatasetIdentifiers.add(binaryDataset.getBinaryDatasetIdentifier());
		}
		ObjectStore.writeObject(this, baos);
		baos.close();
		byte[] byteArray = baos.toByteArray();
		length = byteArray.length;
		lengthComputed = true;
		return new ByteArrayInputStream(byteArray);
	}

	@Override
	public long getBinaryDatasetLength() throws IOException {
		if (!lengthComputed) {
			createInputStream();
		}
		return length;
	}

	@Override
	public void processBinaryDataset(AudioProject localProject, String binaryDatasetIdentifier, InputStream is, long length, final BinaryDatasetProcessCallback callback) throws IOException,
			IllegalAccessException, ClassNotFoundException, InstantiationException, NoSuchMethodException, InvocationTargetException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		readExact(is, baos, length, new ReadUpdateCallback() {
			@Override
			public void readUpdate(long read, long total) {
				callback.binaryDatasetProcessUpdate(read, total);
			}
		});
		baos.close();
		if (localProject != null) {
			ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
			BinaryAudioProject remoteBinaryAudioProject = (BinaryAudioProject) ObjectStore.readObject(bais);
			AudioProject remoteProject = remoteBinaryAudioProject.audioProject;
			remoteProject.initializeAudioDevices();
			localProject.synchronizeBy(remoteProject);
			// any binary dataset should be transmitted once, even if its
			// presence is multiple
			additionalBinaryDatasets = new TreeSet<BinaryDataset>(this);
			additionalBinaryDatasets.addAll(localProject.getAllBinaryDatasets());
			Collection<String> remoteBinaryDatasetIdentifiers = remoteBinaryAudioProject.remoteAvailableBinaryDatasetIdentifiers;
			// may be null because empty collections do not become serialized by
			// the ObjectStore
			if (remoteBinaryDatasetIdentifiers != null && !additionalBinaryDatasets.isEmpty()) {
				for (String remoteBinaryDatasetIdentifier : remoteBinaryDatasetIdentifiers) {
					Collection<BinaryDataset> instancesWithSameIdentifierInCollection = findInstancesWithSameIdentifierInCollection(additionalBinaryDatasets, remoteBinaryDatasetIdentifier);
					additionalBinaryDatasets.removeAll(instancesWithSameIdentifierInCollection);
				}
			}
		}
	}

	private static Collection<BinaryDataset> findInstancesWithSameIdentifierInCollection(Collection<BinaryDataset> datasets, String identifier) {
		Collection<BinaryDataset> ret = new HashSet<BinaryDataset>();
		for (BinaryDataset binaryDataset : datasets) {
			if (binaryDataset.getBinaryDatasetIdentifier().equals(identifier)) {
				ret.add(binaryDataset);
			}
		}
		return ret;
	}

	@Override
	public Collection<BinaryDataset> getAdditionalBinaryDatasetsForRemote() {
		return additionalBinaryDatasets;
	}

	@Override
	public int compare(BinaryDataset b1, BinaryDataset b2) {
		return b1.getBinaryDatasetIdentifier().compareTo(b2.getBinaryDatasetIdentifier());
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/model/BinaryAudioProject.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import static org.audiochain.model.ObjectStore.readExact;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.TreeSet;

import org.audiochain.model.ObjectStore.ReadUpdateCallback;


public final class BinaryAudioProject implements BinaryDataset, RemoteBinaryDatasetProvider, Serializable, Comparator<BinaryDataset> {
	private static final long serialVersionUID = 1L;
	private final AudioProject audioProject;
	private transient long length;
	private transient boolean lengthComputed;
	private transient Collection<BinaryDataset> additionalBinaryDatasets;
	private Collection<String> remoteAvailableBinaryDatasetIdentifiers;

	public BinaryAudioProject() {
		// for deserialization only
		audioProject = null;
	}

	public BinaryAudioProject(AudioProject audioProject) {
		this.audioProject = audioProject;
	}

	@Override
	public AudioProject getBinaryDatasetAssociatedAudioProject() {
		return audioProject;
	}

	@Override
	public String getBinaryDatasetIdentifier() {
		return audioProject.getIdentifier();
	}

	@Override
	public InputStream getBinaryDatasetInputStream() throws IOException {
		return createInputStream();
	}

	private InputStream createInputStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		remoteAvailableBinaryDatasetIdentifiers = new HashSet<String>();
		Collection<BinaryDataset> allBinaryDatasets = audioProject.getAllBinaryDatasets();
		for (BinaryDataset binaryDataset : allBinaryDatasets) {
			remoteAvailableBinaryDatasetIdentifiers.add(binaryDataset.getBinaryDatasetIdentifier());
		}
		ObjectStore.writeObject(this, baos);
		baos.close();
		byte[] byteArray = baos.toByteArray();
		length = byteArray.length;
		lengthComputed = true;
		return new ByteArrayInputStream(byteArray);
	}

	@Override
	public long getBinaryDatasetLength() throws IOException {
		if (!lengthComputed) {
			createInputStream();
		}
		return length;
	}

	@Override
	public void processBinaryDataset(AudioProject localProject, String binaryDatasetIdentifier, InputStream is, long length, final BinaryDatasetProcessCallback callback) throws IOException,
			IllegalAccessException, ClassNotFoundException, InstantiationException, NoSuchMethodException, InvocationTargetException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		readExact(is, baos, length, new ReadUpdateCallback() {
			@Override
			public void readUpdate(long read, long total) {
				callback.binaryDatasetProcessUpdate(read, total);
			}
		});
		baos.close();
		if (localProject != null) {
			ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
			BinaryAudioProject remoteBinaryAudioProject = (BinaryAudioProject) ObjectStore.readObject(bais);
			AudioProject remoteProject = remoteBinaryAudioProject.audioProject;
			remoteProject.initializeAudioDevices();
			localProject.synchronizeBy(remoteProject);
			// any binary dataset should be transmitted once, even if its
			// presence is multiple
			additionalBinaryDatasets = new TreeSet<BinaryDataset>(this);
			additionalBinaryDatasets.addAll(localProject.getAllBinaryDatasets());
			Collection<String> remoteBinaryDatasetIdentifiers = remoteBinaryAudioProject.remoteAvailableBinaryDatasetIdentifiers;
			// may be null because empty collections do not become serialized by
			// the ObjectStore
			if (remoteBinaryDatasetIdentifiers != null && !additionalBinaryDatasets.isEmpty()) {
				for (String remoteBinaryDatasetIdentifier : remoteBinaryDatasetIdentifiers) {
					Collection<BinaryDataset> instancesWithSameIdentifierInCollection = findInstancesWithSameIdentifierInCollection(additionalBinaryDatasets, remoteBinaryDatasetIdentifier);
					additionalBinaryDatasets.removeAll(instancesWithSameIdentifierInCollection);
				}
			}
		}
	}

	private static Collection<BinaryDataset> findInstancesWithSameIdentifierInCollection(Collection<BinaryDataset> datasets, String identifier) {
		Collection<BinaryDataset> ret = new HashSet<BinaryDataset>();
		for (BinaryDataset binaryDataset : datasets) {
			if (binaryDataset.getBinaryDatasetIdentifier().equals(identifier)) {
				ret.add(binaryDataset);
			}
		}
		return ret;
	}

	@Override
	public Collection<BinaryDataset> getAdditionalBinaryDatasetsForRemote() {
		return additionalBinaryDatasets;
	}

	@Override
	public int compare(BinaryDataset b1, BinaryDataset b2) {
		return b1.getBinaryDatasetIdentifier().compareTo(b2.getBinaryDatasetIdentifier());
	}
}
